<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

/*
*	My_Model is an extension to CodeIgniter's core model that helps make
*	developing models easier and less repetitive.
*
*	Version 2009.11.21.0
*	Written by Jesse Terry
*/

class MY_Model extends Model {

	var $table_name;

	var $primary_key;

	var $joins;

	var $select_fields;

	var $validation_fields;

	var $validation_rules;

	var $total_rows;

	var $page_links;

	var $current_page;

	var $num_pages;

	var $optional_params;

	var $order_by;

	function MY_Model() {

		parent::Model();

	}

	function _get($params=NULL) {

	// prepare the query segments
		$this->_prep_params($params);

		// set up the joins
		$this->_prep_joins();

		// execute the query
		$query = $this->db->get($this->table_name);

		$this->_prep_pagination($params);

		if (isset($params['where'][$this->primary_key])) {

		// return a single row if the primary key exists in the where element
			return $query->row();

		}

		else {

		// otherwise return a full result set
			return $query->result();

		}

	}

	function _save($db_array, $id=NULL, $set_flashdata = TRUE) {

		$success = FALSE;

		if ($id) {

			$this->db->where($this->primary_key, $id);

			$success = $this->db->update($this->table_name, $db_array);

		}

		else {

			$success = $this->db->insert($this->table_name, $db_array);

		}

		if ($set_flashdata) {

			$this->session->set_flashdata('success_save', TRUE);

		}

		return $success;

	}

	function _delete($params, $set_flashdata = TRUE) {

		foreach ($params as $field=>$value) {

			$this->db->where($field, $value);

		}

		$this->db->delete($this->table_name);

		if ($set_flashdata) {

			$this->session->set_flashdata('success_delete', TRUE);

		}

	}

	function _db_array() {

		$db_array = array();

		foreach (array_keys($this->validation_fields) as $field) {

			if (isset($_POST[$field])) {

				$db_array[$field] = $this->input->post($field);

			}

		}

		return $db_array;

	}

	function _prep_params($params=NULL) {

		if (isset($params['select'])) {

			$this->db->select($params['select'], FALSE);

		}

		elseif (isset($this->select_fields)) {

			$this->db->select($this->select_fields, FALSE);

		}

		if (isset($params['where'])) {

			if (is_array($params['where'])) {

				foreach ($params['where'] as $key=>$value) {

					$this->db->where($key, $value);

				}

			}

			else {

				$this->db->where($params['where']);

			}

		}

		// should the results be paginated?
		if (isset($params['paginate']) AND $params['paginate'] == TRUE AND isset($params['limit'])) {

			if (!isset($params['page'])) {

			// set 1 as the default page if not specified
				$params['page'] = 1;

			}

			// define the offset
			$offset = ($params['page'] * $params['limit']) - $params['limit'];

			// set the limit
			$this->db->limit($params['limit'], $offset);

		}

		elseif (isset($params['limit']) AND (!isset($params['paginate']) OR $params['paginate'] == FALSE)) {

			$this->db->limit($params['limit']);

		}

		if (isset($params['order_by'])) {

			$this->db->order_by($params['order_by']);

		}

		elseif (isset($this->order_by)) {

			$this->db->order_by($this->order_by);

		}

		// are there any optional parameters?

		if (isset($params) AND isset($this->optional_params)) {

			foreach ($this->optional_params as $key=>$param) {

				if (key_exists($key, $params)) {

					$method = $this->optional_params[$key]['method'];

					$clause = $this->optional_params[$key]['clause'];

					$this->db->$method($clause);

				}

			}

		}

	}

	function _prep_pagination($params) {

		if (isset($params['paginate']) AND $params['paginate'] == TRUE) {

			$query = $this->db->query('SELECT FOUND_ROWS() AS total_rows');

			$this->total_rows = $query->row()->total_rows;

			$this->load->library('pagination');

			$config = array(
				'base_url'		=>	$this->lib_pagination->base_url(),
				'total_rows'	=>	$this->total_rows,
				'per_page'		=>	$params['limit']);

			if (isset($params['page'])) {

				$config['cur_page'] = $params['page'];

			}

			else {

				$config['cur_page'] = 1;

			}

			$this->pagination->initialize($config);

			$this->page_links = $this->pagination->create_links();

			$this->current_page = $config['cur_page'];

			$this->num_pages = ceil($this->total_rows / $params['limit']);

		}

	}

	function _prep_joins() {

		if (isset($this->joins)) {
			
			foreach ($this->joins as $table=>$join) {

				if (is_array($join)) {

					$this->db->join($table, $join[0], $join[1]);

				}

				else {

					$this->db->join($table, $join);

				}

			}

		}

	}

	function _validate() {

		$this->validation->set_fields($this->validation_fields);

		$this->validation->set_rules($this->validation_rules);

		if ($this->validation->run()) {

			return TRUE;

		}

		else {

			return FALSE;

		}

	}

	function _prep_validation($id) {

	// this function will return the initial values to populate a form on an edit

		$result = $this->_get(array('where'=>array($this->primary_key=>$id)));

		foreach (array_keys($this->validation_fields) as $field) {

			if (isset($result->$field)) {

				$this->validation->$field = $result->$field;

			}

		}

	}

}

?>