<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Invoices extends Admin_Controller {

	function __construct() {

		parent::__construct();

		$this->load->model('mdl_invoices');

		/*
		 * Notify the settings module that we have a save view
		 */
		$this->kindle->set_notice('settings_core_view', array('module'=>'invoices', 'view'=>'settings'));

		/*
		 * Notify the settings module that we have a save method
		 */
		$this->kindle->set_notice('settings_core_save', array('module'=>'invoices', 'method'=>'save_settings'));

	}

	function index() {

		/*
		 * Catch specific post activities with this handler.
		 */
		$this->_post_handler();

		$this->redir->set_last_index();

		/*
		 * Check to see if a uri segment is set to order results by.
		 */
		if (uri_assoc('order_by') == 'invoice_id') {

			$order_by = 'mcb_invoices.invoice_id DESC';

		}

		elseif (uri_assoc('order_by') == 'client') {

			$order_by = 'to_last_name, to_first_name';

		}

		elseif (uri_assoc('order_by') == 'total') {

			$order_by = 'invoice_total DESC';

		}

		elseif (uri_assoc('order_by') == 'balance') {

			$order_by = 'invoice_balance DESC';

		}

		else {

			/*
			 * The uri segment is not set, so order by default.
			 */
			$order_by = 'mcb_invoices.date_entered DESC, mcb_invoices.invoice_id DESC';

		}

		if (uri_assoc('client_id')) {

			/*
			 * Grab invoices based on client_id uri segment.
			 */
			$invoices = $this->mdl_invoices->get(array('limit'=>20, 'paginate'=>TRUE, 'page'=>uri_assoc('page'), 'where'=>array('client_id'=>uri_assoc('client_id')), 'order_by'=>$order_by));

		}

		elseif (uri_assoc('status') == 'open') {

			/*
			 * Grab only open invoices.
			 */
			$invoices = $this->mdl_invoices->get(array('limit'=>20, 'paginate'=>TRUE, 'page'=>uri_assoc('page'), 'with_balance'=>TRUE, 'order_by'=>$order_by));

		}

		elseif (uri_assoc('status') == 'closed') {

			/*
			 * Grab only closed invoices.
			 */
			$invoices = $this->mdl_invoices->get(array('limit'=>20, 'paginate'=>TRUE, 'page'=>uri_assoc('page'), 'without_balance'=>TRUE, 'order_by'=>$order_by));

		}

		else {

			/*
			 * No restrictions - grab all invoices.
			 */
			$invoices = $this->mdl_invoices->get(array('limit'=>20, 'paginate'=>TRUE, 'page'=>uri_assoc('page'), 'order_by'=>$order_by));

		}

		/*
		 * Prepare data for the view.
		 */
		$data = array(
			'invoices'	    =>	$invoices,
			'page_links'    =>	TRUE,
			'sort_links'	=>	TRUE
		);

		/*
		 * Load the view.
		 */
		$this->load->view('index', $data);

	}

	function create() {

		if (uri_assoc('client_id')) {

			$client_id = uri_assoc('client_id');

		}

		elseif ($this->input->post('client_id')) {

			$client_id = $this->input->post('client_id');

		}

		if (isset($client_id)) {

			$this->mdl_invoices->save($client_id, $this->input->post('date_entered'));

			$invoice_id = $this->db->insert_id();

			redirect('invoices/edit/invoice_id/' . $invoice_id);

		}

		else {

			$this->load->module('clients');

			$this->load->helper('text');

			$data = array(
				'clients'			=>	$this->clients->get(),
				'client_id_length'	=>	$this->clients->client_id_length()
			);

			$this->load->view('choose_client', $data);

		}

	}

	function form() {

		/*
		 * Catch specific post activities with this handler.
		 */
		$this->_post_handler();

		$this->load->helper('form');

		if (!$this->mdl_invoices->_validate()) {

			if (!$_POST AND uri_assoc('invoice_id')) {

				$this->mdl_invoices->_prep_validation(uri_assoc('invoice_id'));

			}

			$this->load->view('form', $data);

		}

		else {

			$this->mdl_invoices->save($this->mdl_invoices->db_array(), uri_assoc('invoice_id'));

			redirect('invoices/index');

		}

	}

	function delete() {

		if (uri_assoc('invoice_id')) {

			$this->mdl_invoices->delete(array('invoice_id'=>uri_assoc('invoice_id')));

		}

		redirect($this->session->userdata('last_index'));

	}

	function edit() {

		/*
		 * Catch specific post activities with this handler.
		 */
		$this->_post_handler();

		$this->redir->set_last_index();

		$this->load->module('clients');

		$this->load->model('payments/mdl_payments');

		$this->load->model('tax_rates/mdl_tax_rates');

		$this->load->helper('text');

		$data = array(
			'invoice'			=>	$this->mdl_invoices->get(array('where'=>array('mcb_invoices.invoice_id'=>uri_assoc('invoice_id')))),
			'payments'			=>	$this->mdl_payments->_get(array('where'=>array('mcb_payments.invoice_id'=>uri_assoc('invoice_id')))),
			'tax_rates'			=>	$this->mdl_tax_rates->_get(),
			'clients'			=>	$this->clients->get(),
			'client_id_length'	=>	$this->clients->client_id_length()
		);

		$this->load->view('edit', $data);

	}

	function generate_pdf() {

		$this->load->plugin('to_pdf');

		$data = array(
			'invoice'		=>	$this->mdl_invoices->get(array('where'=>array('mcb_invoices.invoice_id'=>uri_assoc('invoice_id'))))
		);

		$html = $this->load->view('invoice_templates/' . $this->session->userdata('default_invoice_template'), $data, TRUE);

		pdf_create($html, 'invoice_' . $data['invoice']->invoice_id, TRUE);

	}

	function generate_html() {

		$data = array(
			'invoice'		=>	$this->mdl_invoices->get(array('where'=>array('mcb_invoices.invoice_id'=>uri_assoc('invoice_id'))))
		);

		$this->load->view('invoice_templates/' . $this->session->userdata('default_invoice_template'), $data);

	}

	function get($params = NULL) {

		return $this->mdl_invoices->get($params);

	}

	function invoice_id_length() {
		
		return $this->mdl_invoices->invoice_id_length();

	}

	function save_settings() {

		/*
		 * As per the kindle->set_notice in the constructor, this function will
		 * execute when the core system settings are saved.
		 */

		$this->mdl_invoices->adjust_next_invoice_id($this->input->post('next_invoice_id'));

		$this->mdl_mcb_data->save('currency_symbol', $this->input->post('currency_symbol'));

		$this->mdl_mcb_data->save('default_date_format', $this->input->post('default_date_format'));

		$this->mdl_mcb_data->save('default_date_format_mask', $this->mdl_invoices->date_formats[$this->input->post('default_date_format')]['mask']);

		$this->mdl_mcb_data->save('default_date_format_picker', $this->mdl_invoices->date_formats[$this->input->post('default_date_format')]['picker']);

		$this->session->set_userdata('currency_symbol', $this->input->post('currency_symbol'));

	}

	function _post_handler() {

		if ($this->input->post('btn_add_new_item')) {

			redirect('invoices/items/form/invoice_id/' . uri_assoc('invoice_id'));

		}

		elseif ($this->input->post('btn_add_payment')) {

			redirect('payments/form/invoice_id/' . uri_assoc('invoice_id'));

		}

		elseif ($this->input->post('btn_add_invoice')) {

			redirect('invoices/create');

		}

		elseif ($this->input->post('btn_cancel')) {

			redirect('invoices/index');

		}

		elseif ($this->input->post('btn_submit_invoice_options')) {

			$this->mdl_invoices->save_invoice_options();

			redirect('invoices/edit/invoice_id/' . uri_assoc('invoice_id'));

		}

	}

}

?>