<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Mdl_Invoices extends MY_Model {

	var $date_formats;

	function Mdl_Invoices() {

		parent::MY_Model();

		$this->table_name = 'mcb_invoices';

		$this->primary_key = 'mcb_invoices.invoice_id';

		$this->order_by = 'mcb_invoices.date_entered DESC';

		$this->validation_fields = array(
			'client_id'		=>	$this->lang->line('client'),
			'user_id'		=>	$this->lang->line('created_by'),
			'date_entered'	=>	$this->lang->line('date_entered'),
			'date_closed'	=>	$this->lang->line('date_closed'),
			'invoice_number'	=>	$this->lang->line('invoice_number'),
			'notes'		=>	$this->lang->line('notes')
		);

		$this->validation_rules = array(
			'client_id'			=>	'required',
			'user_id'			=>	'required',
			'date_entered'		=>	'required',
			'invoice_number'	=>	'required'
		);

		$this->select_fields = "
		SQL_CALC_FOUND_ROWS
		mcb_invoices.*,
	    mcb_clients.company_name AS to_company_name,
	    mcb_clients.last_name AS to_last_name,
	    mcb_clients.first_name AS to_first_name,
	    mcb_clients.address AS to_address,
	    mcb_clients.city AS to_city,
	    mcb_clients.state AS to_state,
	    mcb_clients.zip AS to_zip,
	    mcb_clients.phone_number AS to_phone_number,
	    mcb_users.company_name AS from_company_name,
	    mcb_users.last_name AS from_last_name,
	    mcb_users.first_name AS from_first_name,
	    mcb_users.address AS from_address,
	    mcb_users.city AS from_city,
	    mcb_users.state AS from_state,
	    mcb_users.zip AS from_zip,
	    mcb_users.phone_number AS from_phone_number,
	    IFNULL((SELECT FORMAT(SUM(qty * unit_price), 2) FROM mcb_invoice_items WHERE invoice_id = mcb_invoices.invoice_id),0.00) AS invoice_amount,
		(SELECT CASE discount_type WHEN 1 THEN FORMAT(invoice_amount * (discount_amount / 100), 2) WHEN 2 THEN FORMAT(discount_amount, 2) ELSE 0.00 END) AS invoice_amount_discount,
		(SELECT FORMAT((invoice_amount - invoice_amount_discount) * (tax_rate_percent / 100), 2)) AS invoice_tax,
		(SELECT FORMAT((invoice_amount - invoice_amount_discount) + invoice_tax, 2)) AS invoice_total,
	    IFNULL((SELECT FORMAT(SUM(amount), 2) FROM mcb_payments WHERE invoice_id = mcb_invoices.invoice_id),0.00) AS invoice_payment,
	    (SELECT FORMAT(invoice_total - invoice_payment, 2)) AS invoice_balance";

		$this->joins = array(
			'mcb_clients'		=>	'mcb_clients.client_id = mcb_invoices.client_id',
			'mcb_users'			=>	'mcb_users.user_id = mcb_invoices.user_id',
			'mcb_tax_rates'		=>	'mcb_tax_rates.tax_rate_id = mcb_invoices.tax_rate_id'
		);

		$this->optional_params = array(
			'with_balance'		=>	array('method'=>'having','clause'=>'invoice_balance > 0 OR (invoice_balance = 0 AND invoice_payment = 0)'),
			'without_balance'	=>	array('method'=>'having','clause'=>'invoice_balance <= 0 AND invoice_payment > 0')
		);

		$this->date_formats = array(
			'm/d/Y'		=>	array(
			'key'		=>	'm/d/Y',
			'picker'	=>	'mm/dd/yy',
			'mask'		=>	'99/99/9999'),
			'm/d/y'		=>	array(
			'key'		=>	'm/d/y',
			'picker'	=>	'mm/dd/y',
			'mask'		=>	'99/99/99'),
			'Y/m/d'		=>	array(
			'key'		=>	'Y/m/d',
			'picker'	=>	'yy/mm/dd',
			'mask'		=>	'9999/99/99')
		);

	}

	function db_array() {

		$db_array = $this->_db_array();

		return $db_array;

	}

	function get($params=NULL) {

		$this->_prep_params($params);

		$this->_prep_joins();

		$query = $this->db->get($this->table_name);

		$this->_prep_pagination($params);

		if (isset($params['where'][$this->primary_key])) {

			$result = $query->row();

			$result->invoice_items = $this->get_items(array('invoice_id'=>$result->invoice_id));

			$this->load->model('payments/mdl_payments');

			$result->invoice_payments = $this->mdl_payments->_get(array('where'=>array('mcb_payments.invoice_id'=>$result->invoice_id)));

			return $result;

		}

		else {

			return $query->result();

		}

	}

	function get_items($params = NULL) {

		$this->db->select('*, qty * unit_price AS line_price', FALSE);

		if (isset($params['invoice_item_id'])) {

			$this->db->where('invoice_item_id', $params['invoice_item_id']);

		}

		if (isset($params['invoice_id'])) {

			$this->db->where('invoice_id', $params['invoice_id']);

		}

		if (!isset($params['order_by'])) {

			$this->db->order_by('invoice_item_id');

		}

		$query = $this->db->get('mcb_invoice_items');

		if (isset($params['invoice_item_id'])) {

			return $query->row();

		}

		else {

			return $query->result();

		}

	}

	function save($client_id, $date_entered) {

		$db_array = array(
			'client_id'		=>	$client_id,
			'date_entered'	=>	strtotime($date_entered),
			'user_id'		=>	$this->session->userdata('user_id'),
			'tax_rate_id'	=>	$this->session->userdata('default_tax_rate_id')
		);

		$this->db->insert($this->table_name, $db_array);

	}

	function adjust_status() {

		$this->db->select('
		(SELECT SUM(qty * unit_price) FROM mcb_invoice_items WHERE invoice_id = mcb_invoices.invoice_id) AS invoice_total,
		(SELECT SUM(amount) FROM mcb_payments WHERE invoice_id = mcb_invoices.invoice_id) AS payment_total');

		$this->db->where('invoice_id', uri_assoc('invoice_id'));

		$query = $this->db->get($this->table_name);

		$result = $query->row();

		if ($result->payment_total >= $result->invoice_total) {

			$db_array = array(
				'date_closed'	=>	time()
			);

		}

		else {

			$db_array = array(
				'date_closed'	=>	''
			);

		}

		$this->db->where('invoice_id', uri_assoc('invoice_id'));

		$this->db->update($this->table_name, $db_array);

	}

	function invoice_id_length() {

		$this->db->select('MAX(invoice_id) AS max_invoice_id');

		$query = $this->db->get($this->table_name);

		return strlen($query->row()->max_invoice_id);

	}

	function save_invoice_options() {

		$this->db->where('invoice_id', uri_assoc('invoice_id'));

		$db_array = array(
			'client_id'			=>	$this->input->post('client_id'),
			'date_entered'		=>	strtotime($this->input->post('date_entered')),
			'tax_rate_id'		=>	$this->input->post('tax_rate_id'),
			'discount_amount'	=>	$this->input->post('discount_amount'),
			'discount_type'		=>	$this->input->post('discount_type')
		);

		if (!$this->input->post('discount_amount') or $this->input->post('discount_amount') == '0.00') {

			$db_array['discount_type'] = 0;

		}

		$this->db->update($this->table_name, $db_array);
		
		$this->session->set_flashdata('custom_success', $this->lang->line('invoice_options_saved'));

	}

	function delete($params) {

		$this->db->where('invoice_id', $params['invoice_id']);

		$this->db->delete(array('mcb_invoices', 'mcb_invoice_items', 'mcb_payments'));

	}

	function get_next_invoice_id() {

		$query = $this->db->query("SHOW TABLE STATUS LIKE 'mcb_invoices'");

		return $query->row()->Auto_increment;

	}

	function adjust_next_invoice_id($next_invoice_id) {

		$this->db->query('ALTER TABLE mcb_invoices AUTO_INCREMENT = ' . $next_invoice_id);

	}

}

?>