<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

/*
 * This module is loaded by the parent controllers, so no need to load anywhere else.
 * The main purpose of Kindle is to SIMPLY allow custom modules fit into an application
 * without having to modify the core.
 *
 * Kindle should not be directly accessed via URL. Instead, see mcb_modules
 * controller for usage example.
 */

class Kindle extends Admin_Controller {

	var $notices = array();

	function __construct() {

		parent::__construct();

		$this->load->model('mdl_kindle');

		/*
		 * Kindle will load any custom enabled modules to retrieve their
		 * messages for other modules for triggering actions.
		 */
		$this->_load_custom_modules();

	}

	/*
	 * The index function does a TON of things. I have used a quantum compression
	 * algorithm to hide the code.  Good luck finding it!
	 */
	function index() {}

	/*
	 * 1. Runs the setup/install function of the custom module.
	 * 2. Changes the status of the module to enabled.
	 */
	function install($module_dir) {

		$this->load->module($module_dir . '/setup');

		$this->setup->install();

		$this->db->where('module_directory', $module_dir);

		$db_array = array(
			'module_enabled'	=>	1
		);

		$this->db->update('kindle_modules', $db_array);

	}

	/*
	 * 1. Runs the setup/uninstall function of the custom module.
	 * 2. Changes the status of the module to disabled.
	 */
	function uninstall($module_dir) {

		$this->load->module($module_dir . '/setup');

		$this->setup->uninstall();

		$this->db->where('module_directory', $module_dir);

		$this->db->delete('kindle_modules');

		$this->mdl_kindle->refresh();

	}
	
	/*
	 * Runs the setup/upgrade function of the custom module.
	 */
	function upgrade($module_dir) {

		$this->load->module($module_dir . '/setup');

		$this->setup->upgrade();

		$this->mdl_kindle->refresh();

	}

	/*
	 * Custom modules send notices here, and the core modules, if applicable,
	 * retrieve their notices for module observation.
	 */
	function set_notice($notice_name, $params) {

		$this->notices[$notice_name][] = $params;

	}

	/*
	 * Load enabled custom modules when kindle is loaded
	 */
	function _load_custom_modules() {

		foreach ($this->mdl_kindle->modules->enabled as $module) {

			$this->load->module($module->module_directory);

		}

	}

	/*
	 * Load core modules when required
	 */
	function load_core_modules() {

		

	}

}

?>