<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Mdl_Kindle extends MY_Model {

	var $modules;

	function Mdl_Kindle() {

		parent::MY_Model();

		$this->table_name = 'kindle_modules';

		$this->primary_key = 'kindle_modules.module_id';

		$this->order_by = 'kindle_modules.module_name';

		$this->set_module_vars();

	}

	/*
	 * Refreshes the database list of available modules.
	 * Executed when modules page is visited and when a module is uninstalled.
	 */
	function refresh() {

		/*
		 * Get a list of all module directories.
		 */
		$directories = directory_map(BASEPATH . 'modules/', TRUE);

		foreach ($directories as $directory) {

			/*
			 * See if the module already has a record in the database.
			 */
			$this->db->where('module_directory', $directory);

			$query = $this->db->get($this->table_name);

			if (!$query->num_rows()) {

				/*
				 * Custom modules need a config file.
				 * @todo - Need to do better error handling if config file does not exist.
				 */
				$config = $this->load->config($directory . '/config', TRUE);

				/*
				 * Make sure the required config variables exist.
				 */
				if (isset($config['module_name']) AND isset($config['module_description']) AND isset($config['module_version']) AND isset($config['module_author']) AND isset($config['module_homepage'])) {

					/*
					 * Prepare the array to be inserted into the database.
					 */
					$db_array = array(
						'module_directory'			=>	$directory,
						'module_name'				=>	$config['module_name'],
						'module_description'		=>	$config['module_description'],
						'module_enabled'			=>	0,
						'module_author'				=>	$config['module_author'],
						'module_version'			=>	$config['module_version'],
						'module_available_version'	=>	$config['module_version'],
						'module_homepage'			=>	$config['module_homepage']
					);

					/*
					 * Insert the record!
					 */
					$this->db->insert('kindle_modules', $db_array);

				}

			}

			else {
			
				/*
				 * This module has an entry in the modules table.
				 * Check to see if the module is newer than installed.
				 */
				$config = $this->load->config($directory . '/config', TRUE);

				if ($query->row()->module_enabled) {

					$db_array = array(
						'module_available_version'	=>	$config['module_version']
					);

				}

				else {

					$db_array = array(
						'module_version'			=>	$config['module_version'],
						'module_available_version'	=>	$config['module_version']
					);

				}
				
				$this->db->where('module_directory', $directory);

				$this->db->update('kindle_modules', $db_array);

			}

		}

		$this->set_module_vars();

	}

	function set_module_vars() {

		/*
		 * Populate the list of available modules, both enabled and disabled.
		 * Controllers can refer to $this->mdl_kindle->modules->enabled
		 */

		$this->modules->enabled = array();

		$this->modules->disabled = array();

		$modules = $this->_get();

		foreach ($modules as $module) {

			if ($module->module_enabled == 1) {

				$this->modules->enabled[$module->module_directory] = $module;

			}

			else {

				$this->modules->disabled[$module->module_directory] = $module;

			}

		}

	}

	/*
	 * Should only be called when a module needs to access kindle notices.
	 */
	function load_core_modules() {

		$modules = directory_map(APPPATH . 'modules/', TRUE);

		foreach ($modules as $module) {

			if ($module <> 'kindle' and $module <> 'setup') {

				$this->load->module($module);

			}

		}

	}

	/*
	 * @todo - Might get rid of this function and just check $this->mdl_kindle->modules->enabled instead?
	 */
	function check_enable($mod_dir) {

		foreach ($this->modules->enabled as $module) {

			if ($module->module_directory == $mod_dir) {

				return TRUE;

			}

		}

		return FALSE;

	}

}

?>