<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Mdl_Setup extends MY_Model {

	var $install_version = '0.6.3.2';

	var $upgrade_path;

	function Mdl_Setup() {

		parent::MY_Model();

		$this->table_name = 'mcb_users';

		$this->primary_key = 'mcb_users.user_id';

		$this->validation_fields = array(
			'first_name'    =>	$this->lang->line('first_name'),
			'last_name'     =>	$this->lang->line('last_name'),
			'username'		=>	$this->lang->line('username'),
			'password'		=>	$this->lang->line('password'),
			'passwordv'		=>	$this->lang->line('password_verify'),
			'address'		=>	$this->lang->line('street_address'),
			'city'			=>	$this->lang->line('city'),
			'state'			=>	$this->lang->line('state'),
			'zip'			=>	$this->lang->line('zip'),
			'phone_number'	=>	$this->lang->line('phone_number'),
			'fax_number'	=>	$this->lang->line('fax_number'),
			'mobile_number'	=>	$this->lang->line('mobile_number'),
			'email_address'	=>	$this->lang->line('email_address'),
			'web_address'	=>	$this->lang->line('web_address'),
			'company_name'	=>	$this->lang->line('company_name')
		);

		$this->validation_rules = array(
			'first_name'	=>	'required',
			'last_name'		=>	'required',
			'username'		=>	'required',
			'password'		=>	'required',
			'passwordv'		=>	'required|matches[password]'
		);

		$this->upgrade_path = array(
			array(
			'from'		=>	'0.5.2',
			'to'		=>	'0.5.5',
			'function'	=>	'u0521_to_055'),
			array(
			'from'      =>  '0.5.2.1',
			'to'        =>  '0.5.5',
			'function'  =>  'u0521_to_055'),
			array(
			'from'      =>  '0.5.5',
			'to'        =>  '0.5.5.2',
			'function'  =>  'u055_to_0552'),
			array(
			'from'		=>	'0.5.5.2',
			'to'		=>	'0.6.0',
			'function'	=>	'u0552_to_060'),
			array(
			'from'		=>	'0.6.0',
			'to'		=>	'0.6.1',
			'function'	=>	'u060_to_061'),
			array(
			'from'		=>	'0.6.1',
			'to'		=>	'0.6.2.4',
			'function'	=>	'u061_to_0624'),
			array(
			'from'		=>	'0.6.2.4',
			'to'		=>	'0.6.2.5',
			'function'	=>	'u0624_to_0625'),
			array(
			'from'		=>	'0.6.2.5',
			'to'		=>	'0.6.2.6',
			'function'	=>	'u0625_to_0626'),
			array(
			'from'		=>	'0.6.2.6',
			'to'		=>	'0.6.3',
			'function'	=>	'u0626_to_063'),
			array(
			'from'		=>	'0.6.3',
			'to'		=>	'0.6.3.1',
			'function'	=>	'u063_to_0631'),
			array(
			'from'		=>	'0.6.3.1',
			'to'		=>	'0.6.3.2',
			'function'	=>	'u0631_to_0632')
		);

	}

	function db_array() {

		$db_array = $this->_db_array();

		unset($db_array['passwordv']);

		$db_array['password'] = md5($db_array['password']);

		$db_array['global_admin'] = 1;

		return $db_array;

	}

	function db_install() {

		$return = array();

		$this->load->database();

		$this->db->db_debug = 0;

		if ($this->db_install_tables()) {

			$return[] = $this->lang->line('install_database_success');

		}

		else {

			$return[] = $this->lang->line('install_database_problem');

			return $return;

		}

		if ($this->mdl_setup->_save($this->mdl_setup->db_array(), NULL, FALSE)) {

			$return[] = $this->lang->line('install_admin_account_success');

		}

		else {

			$return[] = $this->lang->line('install_admin_account_problem');

			return $return;

		}

		$return[] = $this->lang->line('installation_complete');

		$return[] = $this->lang->line('install_delete_folder');

		$return[] = APPPATH . 'modules/setup';

		$return[] = anchor('sessions/login', $this->lang->line('log_in'));

		return $return;

	}

	function db_install_tables() {

		foreach ($this->db_tables() as $query) {

			if (!$this->db->query($query)) {

				return FALSE;

			}

		}

		$this->mdl_mcb_data->save('version', $this->install_version);

		return TRUE;

	}

	function db_tables() {

		return array(
		"CREATE TABLE IF NOT EXISTS `mcb_clients` (
		`client_id` int(11) NOT NULL auto_increment,
		`username` varchar(25) NOT NULL,
		`password` varchar(50) NOT NULL,
		`first_name` varchar(50) NOT NULL,
		`last_name` varchar(50) NOT NULL,
		`address` varchar(100) NOT NULL,
		`city` varchar(50) NOT NULL,
		`state` varchar(50) NOT NULL,
		`zip` varchar(10) NOT NULL,
		`phone_number` varchar(25) NOT NULL,
		`fax_number` varchar(25) NOT NULL,
		`mobile_number` varchar(25) NOT NULL,
		`email_address` varchar(100) NOT NULL,
		`web_address` varchar(255) NOT NULL,
		`company_name` varchar(255) NOT NULL,
		`notes` longtext NOT NULL,
		PRIMARY KEY  (`client_id`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_invoices` (
		`invoice_id` int(11) NOT NULL auto_increment,
		`client_id` int(11) NOT NULL,
		`user_id` int(11) NOT NULL,
		`date_entered` varchar(25) NOT NULL,
		`date_closed` varchar(25) NOT NULL,
		`invoice_number` varchar(50) NOT NULL,
		`notes` longtext NOT NULL,
		`tax_rate_id` int(11) NOT NULL,
		`discount_amount` decimal(10,2) NOT NULL,
		`discount_type` int(1) NOT NULL,
		PRIMARY KEY  (`invoice_id`),
		KEY `client_id` (`client_id`),
		KEY `user_id` (`user_id`),
		KEY `tax_rate_id` (`tax_rate_id`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_invoice_items` (
		`invoice_item_id` int(11) NOT NULL auto_increment,
		`invoice_id` int(11) NOT NULL,
		`item` longtext NOT NULL,
		`qty` decimal(10,2) NOT NULL,
		`unit_price` decimal(10,2) NOT NULL,
		PRIMARY KEY  (`invoice_item_id`),
		KEY `invoice_id` (`invoice_id`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_data` (
		`mcb_data_id` int(11) NOT NULL auto_increment,
		`mcb_key` varchar(50) NOT NULL,
		`mcb_value` varchar(100) NOT NULL,
		PRIMARY KEY  (`mcb_data_id`),
		UNIQUE KEY `mcb_data_key` (`mcb_key`)
		) ENGINE=MyISAM  DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_payments` (
		`payment_id` int(11) NOT NULL auto_increment,
		`invoice_id` int(11) NOT NULL,
		`payment_date` varchar(25) NOT NULL,
		`amount` decimal(10,2) NOT NULL,
		`note` longtext NOT NULL,
		PRIMARY KEY  (`payment_id`),
		KEY `invoice_id` (`invoice_id`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_users` (
		`user_id` int(11) NOT NULL auto_increment,
		`username` varchar(25) NOT NULL,
		`password` varchar(50) NOT NULL,
		`first_name` varchar(50) NOT NULL,
		`last_name` varchar(50) NOT NULL,
		`address` varchar(100) NOT NULL,
		`city` varchar(50) NOT NULL,
		`state` varchar(50) NOT NULL,
		`zip` varchar(10) NOT NULL,
		`phone_number` varchar(25) NOT NULL,
		`fax_number` varchar(25) NOT NULL,
		`mobile_number` varchar(25) NOT NULL,
		`email_address` varchar(100) NOT NULL,
		`web_address` varchar(255) NOT NULL,
		`company_name` varchar(255) NOT NULL,
		`last_login` varchar(25) NOT NULL,
		`global_admin` int(1) NOT NULL default '0',
		PRIMARY KEY  (`user_id`)
		) ENGINE=MyISAM  DEFAULT CHARSET=utf8;",

		"CREATE TABLE IF NOT EXISTS `mcb_tax_rates` (
		`tax_rate_id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
		`tax_rate_name` VARCHAR( 25 ) NOT NULL ,
		`tax_rate_percent` DECIMAL( 5, 2 ) NOT NULL
		) ENGINE = MYISAM DEFAULT CHARSET=utf8 ;",

		"CREATE TABLE IF NOT EXISTS `kindle_modules` (
		  `module_id` int(11) NOT NULL AUTO_INCREMENT,
		  `module_directory` varchar(50) NOT NULL,
		  `module_name` varchar(50) NOT NULL,
		  `module_description` varchar(255) NOT NULL,
		  `module_enabled` int(1) NOT NULL,
		  `module_author` varchar(50) NOT NULL,
		  `module_homepage` varchar(255) NOT NULL,
		  `module_version` varchar(25) NOT NULL,
		  `module_available_version` varchar(25) NOT NULL,
		  `module_settings_view` varchar(50) NOT NULL,
		  `module_settings_function` varchar(50) NOT NULL,
		  `module_dashboard_function` varchar(50) NOT NULL,
		  PRIMARY KEY (`module_id`)
		) ENGINE=MyISAM  DEFAULT CHARSET=utf8;",

		"INSERT INTO `mcb_tax_rates` (
		`tax_rate_name` ,
		`tax_rate_percent`	)
		VALUES (
		'No Tax',
		'0.00');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'currency_symbol',
		'$');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'default_tax_rate_id',
		'1');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'dashboard_show_open_invoices',
		'TRUE');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'dashboard_show_closed_invoices',
		'TRUE');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'default_date_format',
		'm/d/Y');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'default_date_format_mask',
		'99/99/9999');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'default_date_format_picker',
		'mm/dd/yy');",

		"INSERT INTO `mcb_data` (
		`mcb_key` ,
		`mcb_value`	)
		VALUES (
		'default_invoice_template',
		'default');"

		);

	}

	function db_upgrade() {

		$this->load->database();

		$return = array();

		if ($this->mdl_mcb_data->get('version') <> $this->install_version) {

			foreach ($this->upgrade_path as $path) {

				$app_version = $this->mdl_mcb_data->get('version');

				if ($path['from'] == $app_version) {

					if ($this->{$path['function']}()) {

						$return[] = 'Upgrade from ' . $path['from'] . ' to ' . $path['to'] . ' successful<br />';

					}

					else {

						$return[] = 'Upgrade from ' . $path['from'] . ' to ' . $path['to'] . ' FAILED. Script exiting.';

						return $return;

					}

				}

			}

			$return[] = $this->lang->line('upgrade_complete');

			$return[] = $this->lang->line('install_delete_folder');

			$return[] = APPPATH . 'modules/setup';

			$return[] = anchor('sessions/login', $this->lang->line('log_in'));

			return $return;

		}

		else {

			$return[] = anchor('sessions/login', $this->lang->line('log_in'));

			$return[] = $this->lang->line('install_already_current');

			return $return;

		}

	}

	function u0521_to_055() {

		$queries = array(
			'RENAME TABLE `clients` TO `mcb_clients`;',
			'RENAME TABLE `invoices` TO `mcb_invoices`;',
			'RENAME TABLE `invoice_items` TO `mcb_invoice_items`;',
			'RENAME TABLE `payments` TO `mcb_payments`;',
			'RENAME TABLE `users` TO `mcb_users`;'
		);

		return $this->run_update($queries, '0.5.5');

	}

	function u055_to_0552() {

		$this->mdl_mcb_data->save('version', '0.5.5.2');

		return TRUE;

	}

	function u0552_to_060() {

		$queries = array(
			"ALTER TABLE `mcb_invoices` ADD `tax_rate_id` INT NOT NULL , ADD INDEX ( tax_rate_id )",
			"CREATE TABLE IF NOT EXISTS `mcb_tax_rates` (
			`tax_rate_id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
			`tax_rate_name` VARCHAR( 25 ) NOT NULL ,
			`tax_rate_percent` DECIMAL( 5, 2 ) NOT NULL
			) ENGINE = MYISAM DEFAULT CHARSET=utf8;",
			"INSERT INTO `mcb_tax_rates` (
			`tax_rate_name` ,
			`tax_rate_percent`	)
			VALUES (
			'No Tax',
			'0.00');");

		$this->mdl_mcb_data->save('default_tax_rate_id', 1);

		$this->mdl_mcb_data->save('currency_symbol', '$');

		$db_array = array(
			'tax_rate_id'	=>	1
		);

		$this->db->update('mcb_invoices', $db_array);

		return $this->run_update($queries, '0.6.0');

	}

	function u060_to_061() {

		$queries = array(
			"ALTER TABLE `mcb_payments` CHANGE `date_entered` `payment_date` VARCHAR( 25 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL",
			"DELETE FROM mcb_invoice_items WHERE invoice_id NOT IN (SELECT invoice_id FROM mcb_invoices)",
			"DELETE FROM mcb_payments WHERE invoice_id NOT IN (SELECT invoice_id FROM mcb_invoices)"
		);

		return $this->run_update($queries, '0.6.1');

	}

	function u061_to_0624() {

		$queries = array(
			"CREATE TABLE IF NOT EXISTS `kindle_modules` (
			`module_id` int(11) NOT NULL AUTO_INCREMENT,
			`module_directory` varchar(50) NOT NULL,
			`module_name` varchar(50) NOT NULL,
			`module_description` varchar(255) NOT NULL,
			`module_enabled` int(1) NOT NULL,
			`module_author` varchar(50) NOT NULL,
			`module_homepage` varchar(255) NOT NULL,
			`module_version` varchar(25) NOT NULL,
			PRIMARY KEY (`module_id`)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8;",

			"ALTER TABLE `mcb_data` CHANGE `mcb_key` `mcb_key` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
			CHANGE `mcb_value` `mcb_value` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL");

		if (!$this->run_update($queries, '0.6.2.4')) {

			return FALSE;

		}

		$this->mdl_mcb_data->save('dashboard_show_open_invoices', 'TRUE');

		$this->mdl_mcb_data->save('dashboard_show_closed_invoices', 'TRUE');

		return TRUE;

	}

	function u0624_to_0625() {

		$queries = array(
			"ALTER TABLE `kindle_modules` ADD `module_available_version` VARCHAR( 25 ) NOT NULL"
		);

		return $this->run_update($queries, '0.6.2.5');

	}

	function u0625_to_0626() {

		$this->mdl_mcb_data->save('version', '0.6.2.6');

		return TRUE;

	}

	function u0626_to_063() {

		$this->mdl_mcb_data->save('default_date_format', 'm/d/Y');

		$this->mdl_mcb_data->save('default_date_format_mask', '99/99/9999');

		$this->mdl_mcb_data->save('default_date_format_picker', 'mm/dd/yy');

		$this->mdl_mcb_data->save('default_invoice_template', 'default');

		$this->mdl_mcb_data->save('version', '0.6.3');

		return TRUE;

	}

	function u063_to_0631() {

		$this->mdl_mcb_data->save('version', '0.6.3.1');

		return TRUE;

	}

	function u0631_to_0632() {

		$queries = array(
			"ALTER TABLE `mcb_invoices` ADD `discount_amount` DECIMAL( 10, 2 ) NOT NULL ,
			ADD `discount_type` INT( 1 ) NOT NULL"
		);

		return $this->run_update($queries, '0.6.3.2');

	}

	function run_update($queries, $version) {

		foreach ($queries as $query) {

			if (!$this->db->query($query)) {

				return FALSE;

			}

		}

		$this->mdl_mcb_data->save('version', $version);

		return TRUE;

	}

}

?>