<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Tasks extends Admin_Controller {

	function __construct() {

		parent::__construct();

		/*
		 * Make sure this module is enabled before letting user access it.
		 */
		if (!$this->mdl_kindle->check_enable('tasks')) {

			redirect('dashboard');

		}

		/*
		 * Load this custom module's language file.
		 */
		$this->lang->load('tasks/tasks');

		/*
		 * Load the module's model.
		 */
		$this->load->model('mdl_tasks');

		/*
		 * Notify the dashboard module that we have a widget for it to load.
		 */
		$this->kindle->set_notice('dashboard_widget', array('module'=>'tasks', 'method'=>'dashboard_widget'));

		/*
		 * Notify the settings module that we have a custom save view
		 */
		$this->kindle->set_notice('settings_custom_view', array('module'=>'tasks', 'view'=>'settings'));

		/*
		 * Notify the settings module that we have a custom save method
		 */
		$this->kindle->set_notice('settings_custom_save', array('module'=>'tasks', 'method'=>'save_settings'));

		/*
		 * Notify the dashboard that we have a custom menu to be loaded
		 */
		$this->kindle->set_notice('dashboard_custom_menu', array('module'=>'tasks', 'view'=>'header_menu'));

	}

	function index() {

		/*
		 * Check for form submission and handle appropriately.
		 */
		$this->_post_handler();

		/*
		 * Define this as the last index visited for redirection after edit
		 * or delete.
		 */
		$this->redir->set_last_index();

		/*
		 * Prepare data for the view.
		 */
		$data = array(
			'tasks' =>	$this->mdl_tasks->_get(array('limit'=>20, 'paginate'=>TRUE, 'page'=>uri_assoc('page', 3))),
		);

		/*
		 * Load the view.
		 */
		$this->load->view('index', $data);

	}

	function form() {

		/*
		 * Check for form submission and handle appropriately.
		 */
		$this->_post_handler();

		if (!$this->mdl_tasks->_validate()) {

			/*
			 * The form has not validated, so let's do a few things and
			 * display the form.
			 */

			 /*
			  * Load the form helper.
			  */
			$this->load->helper('form');

			/*
			 * Load the text helper
			 */
			$this->load->helper('text');

			/*
			 * Load the clients model since a list of clients is required
			 * to display on the form.
			 */
			$this->load->model('clients/mdl_clients');

			if (!$_POST AND uri_assoc('task_id', 3)) {

				/*
				 * The form has not been submitted, and this is a record edit.
				 * Prepare validation variables to populate the form.
				 */
				$this->mdl_tasks->prep_validation(uri_assoc('task_id', 3));

			}

			elseif (!$_POST AND !uri_assoc('task_id', 3)) {

				/*
				 * This is a new record, so assign the current date as a
				 * default value for the start date.
				 */
				$this->validation->start_date = date($this->session->userdata('default_date_format'), time());

			}

			/*
			 * Prepare data for the view.
			 */
			$data = array(
				'clients'			=>	$this->mdl_clients->_get()
			);

			/*
			 * Load the view.
			 */
			$this->load->view('form', $data);

		}

		else {

			/*
			 * Yippee!  The form has successfully validated, so let's save.
			 */
			$this->mdl_tasks->save();

			if ($this->input->post('btn_submit_and_create_invoice')) {

				/*
				 * The button was pressed to submit the task and create an
				 * invoice based on the task.
				 */

				$invoice_id = $this->mdl_tasks->create_invoice_from_task();

				/*
				 * Since a new invoice was created, redirect to the invoice.
				 */
				redirect('invoices/edit/invoice_id/' . $invoice_id);

			}

			/*
			 * If the previous block of code was not executed, the user
			 * did not press the button to create an invoice based on the
			 * task, so let's just redirect back to the tasks page.
			 */
			$this->redir->redirect('tasks');

		}

	}

	function delete() {

		/*
		 * First make sure a task has been specified to delete.
		 */
		if (uri_assoc('task_id', 3)) {

			/*
			 * The uri segment variable exists, so delete the task.
			 */
			$this->mdl_tasks->delete(array('task_id'=>uri_assoc('task_id', 3)));

		}

		$this->redir->redirect('tasks');

	}

	function save_settings() {

		/*
		 * As per the kindle->set_notice in the constructor, this function will
		 * execute when the core system settings are saved.
		 */

		if ($this->input->post('dashboard_show_open_tasks')) {

			$this->mdl_mcb_data->save('dashboard_show_open_tasks', "TRUE");

		}

		else {

			$this->mdl_mcb_data->save('dashboard_show_open_tasks', "FALSE");

		}

	}

	function dashboard_widget() {

		/*
		 * As per the module config file, this function will execute when
		 * the core dashboard calls for it.
		 */

		if ($this->mdl_mcb_data->get('dashboard_show_open_tasks') == "TRUE") {

			/*
			 * Only execute if the module's custom system setting has been saved
			 * as TRUE.
			 */
			$data = array(
				'tasks'				=>	$this->mdl_tasks->_get(array('limit'=>10, 'where'=>array('complete_date'=>'0'))),
				'hide_page_links'	=>	TRUE
			);

			$this->load->view('dashboard_widget', $data);

		}

	}

	function _post_handler() {

		/*
		 * Has the Add Task button been pressed?
		 */
		if ($this->input->post('btn_add_task')) {

			redirect('tasks/form');

		}

		/*
		 * Has the Cancel button been pressed?
		 */
		elseif ($this->input->post('btn_cancel')) {

			redirect('tasks/index');

		}

	}

}

?>