<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

class Mdl_Tasks extends MY_Model {

	function Mdl_Tasks() {

		/*
		 * MY_Model does a lot of legwork for us.
		 */
		parent::MY_Model();

		/*
		 * Define the database table name.
		 */
		$this->table_name = 'mcb_tasks';

		/*
		 * Define the primary key of the table.
		 */
		$this->primary_key = 'mcb_tasks.task_id';

		/*
		 * Define SQL_CALC_FOUND_ROWS * as a minimum for pagination.
		 */
		$this->select_fields = "
		SQL_CALC_FOUND_ROWS *";

		/*
		 * Define the default sort order.
		 */
		$this->order_by = 'mcb_tasks.due_date, mcb_tasks.task_id DESC';

		/*
		 * Define the joins.
		 */
		$this->db->joins = array(
			'mcb_clients'	=>	'mcb_clients.client_id = tasks.client_id',
			'mcb_users'		=>	'mcb_users.user_id = tasks.user_id'
		);

		/*
		 * Define the validation fields used for form submissions.
		 */
		$this->validation_fields = array(
			'client_id'		=>	$this->lang->line('client'),
			'start_date'	=>	$this->lang->line('start_date'),
			'due_date'		=>	$this->lang->line('due_date'),
			'complete_date'	=>	$this->lang->line('complete_date'),
			'title'			=>	$this->lang->line('title'),
			'description'	=>	$this->lang->line('description')
		);

		/*
		 * Define the validation rules used for form submissions.
		 */
		$this->validation_rules = array(
			'client_id'		=>	'required',
			'start_date'	=>	'required',
			'title'			=>	'required',
			'description'	=>	'required'
		);

	}

	function save() {

		/*
		 * Prepare the default $db_array
		 */
		$db_array = $this->_db_array();

		/*
		 * If due_date exists, convert it to a unix timestamp
		 */
		if (isset($db_array['due_date'])) {

			$db_array['due_date'] = strtotime($db_array['due_date']);

		}

		/*
		 * If complete_date exists, convert it to a unix timestamp
		 */
		if (isset($db_array['complete_date'])) {

			$db_array['complete_date'] = strtotime($db_array['complete_date']);

		}

		/*
		 * Convert the required start_date to a unix timestamp
		 */
		$db_array['start_date'] = strtotime($db_array['start_date']);

		/*
		 * Is this an existing record or a new record?
		 */
		if (uri_assoc('task_id', 3)) {

			/*
			 * Update this record since it already exists.
			 */
			$this->db->where('task_id', uri_assoc('task_id', 3));

			$this->db->update($this->table_name, $db_array);

		}

		else {

			/*
			 * Insert this record since it's new.
			 */
			$db_array['user_id'] = $this->session->userdata('user_id');

			$this->db->insert($this->table_name, $db_array);

		}

		/*
		 * Tell the view to display a status message
		 */
		$this->session->set_flashdata('success_save', TRUE);

	}

	function create_invoice_from_task() {

		/*
		 * Is this an existing task?
		 */
		if (!uri_assoc('task_id', 3)) {

			$task_id = $this->db->insert_id();

		}

		elseif (uri_assoc('task_id', 3)) {

			$task_id = uri_assoc('task_id', 3);

		}

		/*
		 * Retrieve the task record.
		 */
		$task = $this->_get(array('where'=>array('mcb_tasks.task_id'=>$task_id)));

		/*
		 * Populate the array to insert as an invoice.
		 */
		$db_array = array(
			'client_id'		=>	$task->client_id,
			'user_id'		=>	$task->user_id,
			'date_entered'	=>	$task->start_date,
			'tax_rate_id'	=>	$this->session->userdata('default_tax_rate_id')
		);

		$this->db->insert('mcb_invoices', $db_array);

		/*
		 * Get the newly created invoice_id.
		 */
		$invoice_id = $this->db->insert_id();

		/*
		 * Populate the array to insert as an invoice item.
		 */
		$db_array = array(
			'invoice_id'	=>	$invoice_id,
			'item'			=>	$task->title . ': ' . $task->description,
			'qty'			=>	0,
			'unit_price'	=>	0
		);

		$this->db->insert('mcb_invoice_items', $db_array);

		/*
		 * Populate the array to insert as task-invoice relation.
		 */
		$db_array = array(
			'task_id'		=>	$task_id,
			'invoice_id'	=>	$invoice_id
		);

		$this->db->insert('mcb_tasks_invoices', $db_array);

		/*
		 * Return the invoice_id so the controller can redirect to the newly
		 * created invoice page.
		 */
		return $invoice_id;

	}

	function prep_validation($key) {

		/*
		 * First prepare the default validation
		 */
		$this->_prep_validation($key);

		if (!$_POST) {

			if ($this->validation->due_date) {

				/*
				 * Convert to a human readable date if the unix timestamp exists.
				 */
				$this->validation->due_date = date($this->session->userdata('default_date_format'), $this->validation->due_date);

			}

			if ($this->validation->complete_date) {

				/*
				 * Convert to a human readable date if the unix timestamp exists.
				 */
				$this->validation->complete_date = date($this->session->userdata('default_date_format'), $this->validation->complete_date);

			}

			if ($this->validation->start_date) {

				/*
				 * Convert to a human readable date if the unix timestamp exists.
				 */
				$this->validation->start_date = date($this->session->userdata('default_date_format'), $this->validation->start_date);

			}

		}

	}

	function delete($params) {

		/*
		 * Run the standard delete function.
		 */
		$this->_delete($params);

		/*
		 * And delete records from mcb_tasks_invoices as well.
		 */
		$this->db->where('task_id', $params['task_id']);

		$this->db->delete('mcb_tasks_invoices');

	}

}

?>